﻿using System;
using Code.Entity;
using TMPro;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace Code
{
    public class Controller : MonoBehaviour
    {
        [SerializeField] private Player player;
        [SerializeField] private Enemy enemy;
        [SerializeField] private Button attack;
        [SerializeField] private Button heal;
        [SerializeField] private Button run;
        [SerializeField] private Button defend;
        [SerializeField] private TextMeshProUGUI title;
        [SerializeField] private Image enemyHealth;
        [SerializeField] private Image playerHealth;
        [SerializeField] private TextMeshProUGUI enemyHealthCounter;
        [SerializeField] private TextMeshProUGUI playerHealthCounter;

        private bool _free;
        private float _timer;
        private string _nextAnimation;

        void Start()
        {
            this.attack.onClick.AddListener(PlayerAttack);
            this.heal.onClick.AddListener(PlayerHeal);
            this.run.onClick.AddListener(PlayerRun);
            this.defend.onClick.AddListener(PlayerDefend);
            this.enemyHealthCounter.text = "100%";
            this.playerHealthCounter.text = "100%";
            _free = true;
            _timer = 0;
            title.text = "Your Turn!";
        }

        private void Update()
        {
            if (!_free)
            {
                if (_timer < 1)
                {
                    _timer += Time.deltaTime;
                    return;
                }

                if (_nextAnimation != null)
                {
                    _timer = 0;
                    enemy.PlayAnimation(_nextAnimation);
                    _nextAnimation = null;
                    return;
                }

                this.playerHealth.fillAmount = (float) player.GetHealth() / player.GetMaxHealth();
                this.playerHealthCounter.text = Math.Floor(((float)player.GetHealth() / player.GetMaxHealth()) * 100) + "%";
                    
                _free = true;
                _timer = 0;
            }
        }

        private void PlayerAction(PlayerAction action)
        {
            if (!_free)
            {
                return;
            }

            if (!player.IsAlive())
            {
                SceneManager.LoadScene(0);
            }

            this.title.text = "";
            
            switch (action)
            {
                case Code.PlayerAction.Attack: 
                    player.Attack(this.enemy);
                    this.enemyHealth.fillAmount = (float) enemy.GetHealth() / enemy.GetMaxHealth();
                    this.enemyHealthCounter.text = Math.Floor(((float)enemy.GetHealth() / enemy.GetMaxHealth()) * 100) + "%";
                    break;
                case Code.PlayerAction.Heal: 
                    player.Heal();
                    this.playerHealth.fillAmount = (float)  player.GetHealth() / player.GetMaxHealth();
                    this.playerHealthCounter.text = Math.Floor(((float)player.GetHealth() / player.GetMaxHealth()) * 100) + "%";
                    break;
                case Code.PlayerAction.Defend: 
                    player.SetDefending();
                    break;
                case Code.PlayerAction.Run: 
                    //ToDo run away
                    break;
            }
            
            
            _free = false;
            
            //Play animations 

            if (!enemy.IsAlive())
            {
                SceneManager.LoadScene(0);
                return;
            }
            
            _nextAnimation = this.enemy.HandleTurn(player, title);
            
            Debug.Log("Player: " + player.GetHealth() + "\nEnemy: " + enemy.GetHealth());

            if (enemy.IsAlive()  && player.IsAlive())
            {
                return;
            }

            if (enemy.IsAlive() && !player.IsAlive())//If player is Dead
            {
                Debug.Log("player DEAD");
                SceneManager.LoadScene(0);
                return;
            }

            if (!enemy.IsAlive() && player.IsAlive())//If Enemy is Dead
            {
                Debug.Log("enemy DEAD");
                SceneManager.LoadScene(0);
                //ToDo play win sound or something 
                //Switch-Scene or something
            }
        }

        private void PlayerAttack()
        {
            this.PlayerAction(Code.PlayerAction.Attack);
        }

        private void PlayerHeal()
        {
            this.PlayerAction(Code.PlayerAction.Heal);
        }

        private void PlayerRun()
        {
            this.PlayerAction(Code.PlayerAction.Run);
        }

        private void PlayerDefend()
        {
            this.PlayerAction(Code.PlayerAction.Defend);
        }
    }
}