﻿using TMPro;
using UnityEngine;
using Random = System.Random;

namespace Code.Entity
{
    public abstract class Enemy : Entity
    {
        [SerializeField] protected int attackFrequency = 10;
        [SerializeField] protected int defendFrequency = 20;
        [SerializeField] protected int healFrequency = 100;
        
        
        protected Animator _animator;

        /// <summary>
        /// This method is used to Damage the Entity. It uses the abstract Method "IsDefending" to check if damaging is possible
        /// </summary>
        /// <param name="damage">Damage Dealt</param>
        /// <param name="isSpecial">Defines if Attack is Special</param>
        /// <returns>if Player is still alive</returns>
        public override bool InflictDamage(int damage, bool isSpecial = false)
        {
            if (!this.IsDefending(isSpecial))
            {
                this.health -= damage;
                _animator.SetTrigger("Hit");
                if (this.health < 0)
                {
                    return false;
                }

                return true;
            }

            return true;
        }

        public abstract string HandleTurn(Entity entity, TextMeshProUGUI text);

        public void PlayAnimation(string animation)
        {
            _animator.SetTrigger(animation);
        }
    }
}