﻿using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.Serialization;
using Random = System.Random;

namespace Code.Entity
{
    public abstract class Entity: MonoBehaviour
    {
        [SerializeField] protected int health = 10;
        [SerializeField] protected int attackPower = 1;
        [SerializeField] protected int healPower = 1;
        [SerializeField] protected float specialFailureRate = 0.1f;
        [SerializeField] protected float defendFailureRate = 0.02f;
        [SerializeField] protected new string name = "Entity";
        [SerializeField] protected int maxHealth = 10;
        protected readonly Random Random = new();

        public abstract void Attack(Entity entity);
        public abstract string UseSpecial(Entity entity);
        public abstract bool IsDefending(bool isSpecial);

        public void Heal()
        {
            this.health += this.healPower;
            if (this.health > maxHealth)
            {
                health = maxHealth; 
            }
        }

        /// <summary>
        /// This method is used to Damage the Entity. It uses the abstract Method "IsDefending" to check if damaging is possible
        /// </summary>
        /// <param name="damage">Damage Dealt</param>
        /// <param name="isSpecial">Defines if Attack is Special</param>
        /// <returns>if Player was damaged</returns>
        public virtual bool InflictDamage(int damage, bool isSpecial = false)
        {
            if (!this.IsDefending(isSpecial))
            {
                this.health -= damage;
                return true;
            }

            return false;
        }

        public bool IsAlive()
        {
            return this.health > 0;
        }

        public string GetName()
        {
            return this.name;
        }

        public int GetHealth()
        {
            return this.health;
        }

        public int GetMaxHealth()
        {
            return this.maxHealth;
        }
    }
}